/**
 * FILENAME :        main_Color_10_Click.cpp          
 *
 * DESCRIPTION :
 *       Color_10_Click / RGB Sensor Library Testing.
 *		 required : Color_10_Click library for MBED OS 6
 *
 * NOTES :
 *       Developped by Villou / LEnsE
 **
 * AUTHOR :    Julien VILLEMEJANE        START DATE :    08/feb/2023
 *
 *       LEnsE / Institut d'Optique Graduate School
 */

#include "mbed.h"
#include "COLOR_10_CLICK.h"
#include "PixelArray.h"
#include "WS2812.h"


#define     WS2812_BUF  300  //mettre trop de led demande trop de courant risque --> toute les diode s'allume en rouge car la tension seuil du rouge est plus faible
PixelArray  px(WS2812_BUF);
// For Nucleo F476 : 3, 12, 9, 12
WS2812 ws(D7, WS2812_BUF, 3, 12, 9, 12);

#define WAIT_TIME_MS 500 
DigitalOut led1(LED1);

/// Create an I2C interface with specific pins : SDA / SCL
I2C                 my_color_i2c(D14, D15); // SDA / SCL

/// Create a digital output for WS2812 RGB Led
DigitalOut          my_led(D9);
/// Create a Color_10_Click module connection with an I2C interface and a Digital Out for the led
Color_10_Click      my_sensor_(&my_color_i2c, &my_led);

//declaration de la led qui éclaire la surface
DigitalOut petite_led(D8,1);


/// Create a 5 integers array to collect R, G, B, clear and IR data from the sensor
int             RGBCir[5] = {0};


int main()
{
    //allume la petite led qui sert à éclairer la surface
    petite_led = 1;

    int k = 0;
    printf("Test.\n");
    /// PowerUp and initialize the module in automode
    my_sensor_.powerUp();
    /// Collect informations about the sensor
    printf("Part ID = %d\r\n", my_sensor_.getPartID());
    printf("Command = %d\r\n", my_sensor_.getCommandValue());

    /// Modify the analog gain of the sensor - 3x by default
    my_sensor_.setGainRGB(COLOR_10_CLICK_GAIN_2x);


    while (true)
    {
        k++;
        led1 = !led1;
        /// Collect R, G, B, Clear and IR data from the sensor
        my_sensor_.readRGBCIRValue(RGBCir);


        unsigned char R = (RGBCir[0]*256)/65536;      //convertion de 16 bits (capteur) à 8 Bits (bandeau)
        unsigned char G = 0.5*(RGBCir[1]*256)/65536;    //0.5 = facteur d'attenuation du vert car on pense que le constructeur 
                                                        //à mis un gain pour que le capteur soit copparable à l'oeil humain
        unsigned char B = (RGBCir[2]*256)/65536;


        printf("[%4d] R=%d / G=%d / B=%d / C=%d / IR = %d \r\n\n ", k, R, G, B, RGBCir[3], RGBCir[4]);      //pour TeraTerm
        // Convertir les valeurs de 0-1 à 0-255 et les mettre dans une variable unsigned int
        unsigned int color = ((int)(R) << 16) | ((int)(G) << 8) | (int)(B);
        
       
        for (int i = 1 ; i < WS2812_BUF; i++) {
            px.Set(i,color); // FF : 255 en hexadecimal et ordre RGB
            
        }     
         ws.write(px.getBuf());

        thread_sleep_for(WAIT_TIME_MS);
    }
}



/*
Aide pour unsigned char : https://stackoverflow.com/questions/25762871/equivalent-for-nop-in-c-for-embedded
doc du bandeau : https://www.digikey.fr/en/datasheets/parallaxinc/parallax-inc-28085-ws2812b-rgb-led-datasheet

*/